@@ -1,8 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
+#include "config.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -19,7 +18,6 @@
 #include "cgroup.h"
 #include "commands.h"
 #include "commands_utils.h"
-#include "config.h"
 #include "log.h"
 #include "lxc.h"
 #include "monitor.h"
@@ -114,7 +112,7 @@ int lxc_wait(const char *lxcname, const char *states, int timeout,
 	}
 
 	TRACE("Retrieved state of container %s", lxc_state2str(state));
-	if (!s[state])
+	if ((state < STOPPED || state > MAX_STATE - 1) || !s[state])
 		return -1;
 
 	return 0;
