@@ -3,13 +3,15 @@
 #ifndef __LXC_STORAGE_H
 #define __LXC_STORAGE_H
 
+#include "config.h"
+
 #include <stdint.h>
 #include <sys/mount.h>
 
-#include <lxc/lxccontainer.h>
+#include "lxc.h"
 
 #if IS_BIONIC
-#include <../include/lxcmntent.h>
+#include "lxcmntent.h"
 #else
 #include <mntent.h>
 #endif
@@ -92,7 +94,7 @@ struct lxc_storage {
 };
 
 /**
- * storage_is_dir : Check whether the roots is a directory. This function will
+ * storage_lxc_is_dir : Check whether the roots is a directory. This function will
  *                  trust the config file. If the config file key
  *                  lxc.rootfs.path is set to <storage type>:<container path>
  *                  the confile parser will have split this into <storage type>
@@ -103,7 +105,7 @@ struct lxc_storage {
  *                  type specifications.  If the <storage type> prefix is not
  *                  detected liblxc will try to detect the storage type.
  */
-__hidden extern bool storage_is_dir(struct lxc_conf *conf);
+__hidden extern bool storage_lxc_is_dir(struct lxc_conf *conf);
 __hidden extern bool storage_can_backup(struct lxc_conf *conf);
 __hidden extern struct lxc_storage *storage_init(struct lxc_conf *conf);
 __hidden extern struct lxc_storage *storage_copy(struct lxc_container *c, const char *cname,
