@@ -3,14 +3,15 @@
 #ifndef __LXC_ARGUMENTS_H
 #define __LXC_ARGUMENTS_H
 
+#include "config.h"
+
 #include <getopt.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <sys/param.h>
 #include <sys/types.h>
 
-#include <lxc/lxccontainer.h>
-
+#include "lxc.h"
 #include "compiler.h"
 
 struct lxc_arguments;
@@ -136,6 +137,7 @@ struct lxc_arguments {
 #define LXC_COMMON_OPTIONS                                                     \
 	    { "name",        required_argument, 0, 'n'         },              \
 	    { "help",        no_argument,       0, 'h'         },              \
+	    { "help",        no_argument,       0, '?'         },              \
 	    { "usage",       no_argument,       0, OPT_USAGE   },              \
 	    { "version",     no_argument,       0, OPT_VERSION },              \
 	    { "quiet",       no_argument,       0, 'q'         },              \
